/*
       Copyright 1989, 1990, 1993 Digital Equipment Corporation
                        All Rights Reserved

Permission to use, copy, and modify this software and its documentation is
hereby granted only under the following terms and conditions.  Both the
above copyright notice and this permission notice must appear in all copies
of the software, derivative works or modified versions, and any portions
thereof, and both notices must appear in supporting documentation.

Users of this software agree to the terms and conditions set forth herein,
and hereby grant back to Digital a non-exclusive, unrestricted, royalty-
free right and license under any changes, enhancements or extensions 
made to the core functions of the software, including but not limited to 
those affording compatibility with other hardware or software 
environments, but excluding applications which incorporate this software.
Users further agree to use their best efforts to return to Digital any
such changes, enhancements or extensions that they make and inform Digital
of noteworthy uses of this software.  Correspondence should be provided
to Digital at:

                      Director of Licensing
                      Western Research Laboratory
                      Digital Equipment Corporation
                      250 University Avenue
                      Palo Alto, California  94301

This software may be distributed (but not offered for sale or transferred
for compensation) to third parties, provided such third parties agree to
abide by the terms and conditions of this notice.

THE SOFTWARE IS PROVIDED "AS IS" AND DIGITAL EQUIPMENT CORP. DISCLAIMS ALL
WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS.   IN NO EVENT SHALL DIGITAL EQUIPMENT
CORPORATION BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.
 */

#if !defined(LINT) && !defined(CODECENTER)
static char rcsid[] = "$Id:";
#endif

/* ip_screen.c								*/

/************************************************************************
 *			Modification History				*
 *									*
 *	16 December 1988	Jeffrey Mogul/DECWRL			*
 *		Created.						*
 *									*
 ************************************************************************/

/*
 * IP Screening mechanism
 *	Basically, just an interface to protocol-independent
 *	stuff in net/gw_screen.c
 *
 */

#include "param.h"

#if defined(BSD) && (BSD >= 199103) 

#include "ioctl.h"
#include "mbuf.h"
#include "protosw.h"
#include "socket.h"
#include "socketvar.h"
#include "uio.h"
#include "dir.h"
#include "proc.h"
#include "user.h"
#include "kernel.h"
#include "systm.h"
#include "../net/if.h"
#include "../net/route.h"
#include "in.h"
#include "in_systm.h"
#include "in_var.h"
#include "ip.h"
#include "ip_icmp.h"
#include "ip_var.h"

#else 

#include "../h/ioctl.h"
#include "../h/mbuf.h"
#include "../h/protosw.h"
#include "../h/socket.h"
#include "../h/socketvar.h"
#include "../h/uio.h"
#include "../h/dir.h"
#include "../h/user.h"
#include "../h/proc.h"
#include "../h/kernel.h"
#include "../h/systm.h"
#include "../net/if.h"
#include "../net/route.h"
#include "../net/af.h"
#include "../netinet/in.h"
#include "../netinet/in_systm.h"
#include "../netinet/in_var.h"
#include "../netinet/ip.h"
#include "../netinet/ip_icmp.h"
#include "../netinet/icmp_var.h"

#endif 

void ip_forward();
void ip_gwbounce();

/*
 * This procedure is instead of ip_forward() from ipintr().
 */

#if defined(BSD) && (BSD >= 199103)

ip_forwardscreen(m, srcrt)
	struct mbuf *m;
	int srcrt;	/* NOTUSED */
{
	gw_forwardscreen(m, NULL, AF_INET, ip_forward, ip_gwbounce);
}

#else

ip_forwardscreen(ip, ifp)
	struct ip *ip;
	struct ifnet *ifp;
{
	gw_forwardscreen(ip, ifp, AF_INET, ip_forward, ip_gwbounce);
}

#endif

/*
 * Called from gateway forwarding code if the packet is rejected
 * and the sender should be notified.
 */

#if defined(BSD) && (BSD >= 199306)

void
ip_gwbounce(pkt)
	struct mbuf *pkt;
{
	icmp_error(pkt, ICMP_UNREACH, ICMP_UNREACH_HOST, 0, 0);
}

#else

#if defined(BSD) && (BSD >= 199103)

void
ip_gwbounce(pkt)
	struct mbuf *pkt;
{
	struct in_addr dest;

	dest.s_addr = 0;
	icmp_error(pkt, ICMP_UNREACH, ICMP_UNREACH_HOST, dest);
}

#else

void
ip_gwbounce(pkt, ifp)
	struct ip *pkt;
	struct ifnet *ifp;
{
	struct in_addr dest;

	dest.s_addr = 0;
	icmp_error(pkt, ICMP_UNREACH, ICMP_UNREACH_HOST, ifp, dest);
}

#endif

#endif
